/*

Copyright _ 2006, Joseph Bergin.

*/

/***********************************/
// SAVING AND RETRIEVING PREFERENCES
/***********************************/

// setup() is run when the body loads.  It checks to see if there is a preference for this widget
// and if so, applies the preference to the widget.

function setup()
{
	if(window.widget)		// always check to make sure that you are running in Dashboard
	{
		// The preferences are retrieved:
		var firstUnit = widget.preferenceForKey("firstUnit");
		var nextUnit = widget.preferenceForKey("nextUnit");
		var unitMeasure = widget.preferenceForKey("unitMeasure");
		var currency = widget.preferenceForKey("currency");
		var separator = widget.preferenceForKey("separator");

		if ((!firstUnit || firstUnit.length == 0)
			|| (!nextUnit || nextUnit.length == 0)
			|| (!unitMeasure || unitMeasure.length == 0)
			|| (!currency || currency.length == 0)
			|| (!separator || separator.length == 0)		
			)  // if the retrieved preferences are empty,
		{								// they are initialized.
			firstUnit = "39";
			nextUnit = "24";
			unitMeasure = "oz";
			currency = "$";
			separator = ".";
			widget.setPreferenceForKey(firstUnit, "firstUnit");
			widget.setPreferenceForKey(nextUnit, "nextUnit");
			widget.setPreferenceForKey(unitMeasure, "unitMeasure");
			widget.setPreferenceForKey(currency, "currency");
			widget.setPreferenceForKey(separator, "separator");
		}
		
		var inputText = document.getElementById("inputText");
		var outputText = document.getElementById("outputText");
		
		inputText.value = "0.0"+unitMeasure;
		outputText.value = currency+"0"+separator+"00";
	}
}

function compute()
{
	document.getElementById("doButton").src = "Images/Button_down.png";
	var inputText = document.getElementById("inputText");
	var firstUnit = widget.preferenceForKey("firstUnit");
	var nextUnit = widget.preferenceForKey("nextUnit");
	var currency = widget.preferenceForKey("currency");
	var separator = widget.preferenceForKey("separator");
	var outputText = document.getElementById("outputText");
	var input = parseFloat(inputText.value.replace(separator,"."));
	var units = widget.preferenceForKey("unitMeasure");
	var finalInput = new String(input);
	finalInput = finalInput.replace(/\./,separator) + units
	var value = parseInt(firstUnit) + 
						((Math.ceil(input) - 1) * 
						parseInt(nextUnit));
	inputText.value = finalInput;
	var rem = value%100;
	var rest = rem>9?rem:"0"+rem;
	outputText.value = input>0
					?currency + Math.floor(value/100) + separator + rest						 			
					:currency+"0"+separator+"00";
}

function buttonUpOut()
{
	document.getElementById("doButton").src = "Images/doButton.png";
}



// doUnits() is called whenever a different measurements unit item 
// is chosen in the widget's oz/gm preferences.  

function doUnits(elem) 
{	
	var units = widget.preferenceForKey("unitMeasure");
	
	switch( parseInt(elem.options[elem.selectedIndex].value) )	
		// find out which option was chosen
	{
		case 1: 					// if option #1 ("oz")	
			units="oz";		
			break;
		case 2:						// if option #2 ("gm")
			units = "gm";		
			break;
	}
	if(window.widget)
	{
		widget.setPreferenceForKey(units,"unitMeasure");	
			// and save the new preference to disk
	}
} 


/*********************************/
// HIDING AND SHOWING PREFERENCES
/*********************************/

// showPrefs() is called when the preferences flipper is clicked upon.  It freezes the front of the widget,
// hides the front div, unhides the back div, and then flips the widget over.
// It also fills the pref fields from saved prefs

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToBack");		
		// freezes the widget so that you can change it without the user noticing
		
	var currency = widget.preferenceForKey("currency");
	if(currency && currency.length > 0)
		document.getElementById("currencyText").value = currency;
		
	
	var separator = widget.preferenceForKey( "separator");
	if(separator && separator.length > 0)
		document.getElementById("separatorText").value = separator;
		

	var firstUnit = widget.preferenceForKey( "firstUnit");
	var asString = new String(firstUnit);
	if(asString && !isNaN(asString))
	{	var work = firstUnit/100;
		var left = Math.floor(work);
		var right = new String( work - left);
		var right = right.substr(right.indexOf(".")+1);
		document.getElementById("firstUnitText").value = left+separator+right;
	}	
		
	var nextUnit = widget.preferenceForKey( "nextUnit");
	asString = new String(nextUnit);
	if(asString && !isNaN(asString))
	{	var work = nextUnit/100;
		var left = Math.floor(work);
		var right = new String( work - left);
		var right = right.substr(right.indexOf(".")+1);
		document.getElementById("nextUnitText").value = left+separator+right;
	}	

	var unitMeasure = widget.preferenceForKey("unitMeasure");
	document.getElementById("unitMeasure").value = (unitMeasure=="oz")?1:2;
	
		
	
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget over	

	document.getElementById('fliprollie').style.display = 'none';  // clean up the front side - hide the circle behind the info button

}


// hidePrefs() is called by the done button on the back side of the widget.  It performs the opposite transition
// as showPrefs() does.

// It als saves the pref values.

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
		
	var currency = document.getElementById("currencyText").value;
	if(currency && currency.length > 0)
		widget.setPreferenceForKey(currency, "currency");
	
	var separator = document.getElementById("separatorText").value;
	if(separator && separator.length > 0)
		widget.setPreferenceForKey(separator, "separator");
		
	var firstUnit = document.getElementById("firstUnitText").value.replace(separator,".");
	if(firstUnit && firstUnit.length > 0 && !isNaN(firstUnit))
		widget.setPreferenceForKey(firstUnit*100, "firstUnit");
		
	var nextUnit = document.getElementById("nextUnitText").value.replace(separator,".");
	if(nextUnit && nextUnit.length > 0 && !isNaN(nextUnit))
		widget.setPreferenceForKey(nextUnit*100, "nextUnit");

	var unitMeasure = document.getElementById("unitMeasure").value;
	widget.setPreferenceForKey(unitMeasure==1?"oz":"gm","unitMeasure");
	compute();
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
}


// PREFERENCE BUTTON ANIMATION (- the pref flipper fade in/out)

var flipShown = false;		// a flag used to signify if the flipper is currently shown or not.


// A structure that holds information that is needed for the animation to run.
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};


// mousemove() is the event handle assigned to the onmousemove property on the front div of the widget. 
// It is triggered whenever a mouse is moved within the bounds of your widget.  It prepares the
// preference flipper fade and then calls animate() to performs the animation.

function mousemove (event)
{
	if (!flipShown)			// if the preferences flipper is not already showing...
	{
		if (animation.timer != null)			// reset the animation timer value, in case a value was left behind
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; 		// set it back one frame
		
		animation.duration = 500;												// animation time, in ms
		animation.starttime = starttime;										// specify the start time
		animation.firstElement = document.getElementById ('flip');		// specify the element to fade
		animation.timer = setInterval ("animate();", 13);						// set the animation function
		animation.from = animation.now;											// beginning opacity (not ness. 0)
		animation.to = 1.0;														// final opacity
		animate();																// begin animation
		flipShown = true;														// mark the flipper as animated
	}
}

// mouseexit() is the opposite of mousemove() in that it preps the preferences flipper
// to disappear.  It adds the appropriate values to the animation data structure and sets the animation in motion.

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}


// animate() performs the fade animation for the preferences flipper. It uses the opacity CSS property to simulate a fade.

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = limit_3(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	
	animation.firstElement.style.opacity = animation.now;
}


// these functions are utilities used by animate()

function limit_3 (a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
    return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}